// -------------------------------------------------------------------------------------
//
// roboFishVC.CPP                                                 Ulrich Mller 21.02.06
//                                                                PCS1700       VC++ 6.0
// Funktionen zum Betrieb des ROBO Interfaces auf Basis der TransferArea
//
// -------------------------------------------------------------------------------------

#include <windows.h>
#include <iostream.h>
#include "roboFishVC.H"

DWORD             res;
FT_HANDLE         usbHandle;
UINT              anzDev;
FT_TRANSFER_AREA* ftDCB;

void InitFish(void) {
	cout      << "---InitFish gestartet ---" << endl;
	cout      << "Version : " << GetLibVersion() << endl;
	
	res       = InitFtLib();
	
	cout      << "Init : " << res << endl;
	
	res       = InitFtUsbDeviceList();
	anzDev    = GetAnzFtUsbDevice();
  usbHandle = GetFtUsbDeviceHandle(0);
	
	cout      << "DeviceTyp    " << GetFtDeviceTyp(usbHandle)        << endl;
	cout      << "SerialNr     " << GetFtSerialNrStrg(usbHandle)     << endl;
	cout      << "Firmware     " << GetFtFirmwareStrg(usbHandle)     << endl;
	cout      << "Manufacturer " << GetFtManufacturerStrg(usbHandle) << endl;
	cout      << "ShortName    " << GetFtShortNameStrg(usbHandle)    << endl;
	cout      << "LongName     " << GetFtLongNameStrg(usbHandle)     << endl;
	cout      << "ENDE : Escape-Taste"                               << endl;

	res                = OpenFtUsbDevice(usbHandle);
	res                = StartFtTransferArea(usbHandle, NULL);
	ftDCB              = GetFtTransferAreaAddress(usbHandle);
	ftDCB->MPWM_Update = 0x01;
}

void FinishFish(void) {
	res  = StopFtTransferArea(usbHandle);
	res  = CloseFtDevice(usbHandle);
	res  = CloseFtLib();
	cout << "--- FINIS ---" << endl;
}

BOOL  GetInput(UCHAR InputNr) {
	return ftDCB->E16_Main[--InputNr] ? TRUE : FALSE;
}

UCHAR GetInputs(void) {
	return ftDCB->E_Main;
}

UINT  GetAnalog(UCHAR AnalogNr) {
	switch(--AnalogNr) {
		case 0:  return ftDCB->AX; break;
		case 1:  return ftDCB->AY;
		default: return 0;
	}
}

UINT  GetVoltage(UCHAR VoltNr) {
	switch(--VoltNr) {
		case 0:  return ftDCB->A1; break;
		case 1:  return ftDCB->A2;
		default: return 0;
	}
}

BOOL  GetIRKey(UCHAR Code, UCHAR KeyNr) {
	KeyNr--;
	switch(Code) {
		case 0:
			return ftDCB->IRKeys16[KeyNr] ? TRUE : FALSE;
			break;
		case 1:
			return ftDCB->IRKeys16Code1[KeyNr] ? TRUE : FALSE;
			break;
		case 2:
			return ftDCB->IRKeys16Code2[KeyNr] ? TRUE : FALSE;
		default:
			return FALSE;
			break;
	}
}

void  SetMotor(UCHAR MotorNr, UCHAR Dir, UCHAR Speed) {
	UCHAR n = --MotorNr << 1;
	ftDCB->M_Main &= ~(3 << n);
	switch(Dir) {
		case 0: break;
		case 1:
			ftDCB->M_Main |= 1 << n;
			ftDCB->MPWM_Main[n]   = --Speed;
			break;
		case 2:
			ftDCB->M_Main |= 2 << n;
			ftDCB->MPWM_Main[n+1] = --Speed;
			break;
	}
}

UCHAR GetMotors(void) {
	return ftDCB->M_Main;
}

void  SetMotors(UCHAR MotorStatus, UINT SpeedStatus){
	UCHAR i, i2, i21;
	UCHAR Speed;
	ftDCB->M_Main = MotorStatus;
	for(i = 0; i < 4; i++) { 
		i2  = i * 2;
		i21 = i2 + 1;
		Speed = (SpeedStatus & (0x0F << (i*4))) >> (i*4);
		ftDCB->MPWM_Main[i2]  = Speed;
		ftDCB->MPWM_Main[i21] = Speed;
	}
}

void SetLamp(unsigned short LampNr, BOOL OnOff, UCHAR Power) {
	UINT n = --LampNr % 8;
	if(OnOff) ftDCB->M_Main |= 1 << n;
	else ftDCB->M_Main      &= ~(1 << n);
	ftDCB->MPWM_Main[n]      = --Power;
}

void  WaitForChange(UCHAR InputNr, UINT NrOfChanges, UCHAR TermNr) {
	UINT INew, IOld;
	InputNr--;
	TermNr--;
	IOld = ftDCB->E16_Main[InputNr];
	do {
		if((TermNr < 8) & ftDCB->E16_Main[TermNr]) return;
		INew = ftDCB->E16_Main[InputNr];
		if(INew != IOld) {IOld = INew; NrOfChanges--;}
	} while(NrOfChanges);
}

void  WaitForInput(UCHAR InputNr, BOOL OnOff) {
	while(GetInput(InputNr) != OnOff);
}

void Pause(UINT mSek) {
	Sleep(mSek);
}

BOOL Finish(void) {
	if(GetAsyncKeyState(VK_ESCAPE) != 0) return TRUE;
	else return FALSE;
}